/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 790 $ $LastChangedDate: 2002-08-31 20:33:21 +0200 (Sat, 31 Aug 2002) $
 *
 * C implementation for Core module.
 */

#include <gont/runtime.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

extern gont_ptroff Core$$Match_failure;
extern gont_ptroff Core$$Null_access;
extern gont_ptroff Core$$End_of_file;

struct gont_string *gont_cstring(const char *s)
{
	return gont_make_string(strlen(s), s);
}

void Core__print_string(struct gont_string *s)
{
	fwrite(s->data, s->len, 1, stdout);
}

void Core__prerr_string(struct gont_string *s)
{
	fwrite(s->data, s->len, 1, stderr);
}

/* TODO: fix this to get longer lines. */
struct gont_string *Core__read_line_f(FILE *f)
{
	char buf[4096];
	char *r;

	r = fgets(buf, sizeof(buf), f);

	if (r == NULL)
		gont_raise(gont_make_union(Core$$End_of_file, NULL));
	
	return gont_cstring(buf);
}

/* TODO: fix this to get longer words. */
struct gont_string *Core__read_word_f(FILE *f)
{
	char buf[4096];

	if (fscanf(f, "%4000s", buf) != 1)
		gont_raise(gont_make_union(Core$$End_of_file, NULL));
	
	return gont_cstring(buf);
}

struct gont_string *Core__read_line(void)
{
	return Core__read_line_f(stdin);
}

struct gont_string *Core__read_string(void)
{
	return Core__read_word_f(stdin);
}

struct gont_string *Core__itoa(int i)
{
	char buf[100];

	sprintf(buf, "%d", i);
	
	return gont_cstring(buf);
}

struct gont_string *Core__ftoa(double *f)
{
	char buf[100];

	sprintf(buf, "%g", *f);
	
	return gont_cstring(buf);
}

void gont_match_failure(void)
{
	gont_raise(gont_make_union(Core$$Match_failure, NULL));
}

void gont_null_access(void)
{
	gont_raise(gont_make_union(Core$$Null_access, NULL));
}

gont_ptroff Core__atoi(struct gont_string *s)
{
	char *p;
	gont_ptroff r;

	if (s->len == 0)
		goto fail;
		
	r = strtol(s->data, &p, 0);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return r;

fail:
	gont_invalid_argument("Core::atoi");
	return -1;	/* never reached */
}

double *Core__atof(struct gont_string *s)
{
	char *p;
	double r;

	if (s->len == 0)
		goto fail;
		
	r = strtod(s->data, &p);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return gont_make_float(r);

fail:
	gont_invalid_argument("Core::atof");
	return NULL;	/* never reached */
}

gont_ptroff gont_streq(struct gont_string *s1, struct gont_string *s2)
{
	return s1->len == s2->len &&
	       memcmp(s1->data, s2->data, s1->len) == 0;
}

gont_ptroff gont_strcmp(struct gont_string *s1, struct gont_string *s2)
{
	int r;
	
	if (s1->len < s2->len) {
		r = memcmp(s1->data, s2->data, s1->len);
		return r == 0 ? -1 : r;
	} else if (s1->len > s2->len) {
		r = memcmp(s1->data, s2->data, s2->len);
		return r == 0 ? 1 : r;
	} else
		return memcmp(s1->data, s2->data, s1->len);
}

struct gont_string *gont_strjoin(struct gont_string *s1, 
			         struct gont_string *s2)
{
	struct gont_string *r = gont_make_string(s1->len + s2->len, NULL);

	memcpy(r->data, s1->data, s1->len);
	memcpy(r->data + s1->len, s2->data, s2->len);

	return r;
}

gont_ptroff gont_id(gont_ptroff x)
{
	return x;
}

gont_ptroff gont_hash_string(struct gont_string *s)
{
	gont_ptroff pos = 0, hash = 0;

	while (pos < s->len) {
		hash += s->data[pos++];
		hash *= 37;
	}

	return hash & 0x7fffffff;
}
