/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 909 $ $LastChangedDate: 2002-09-08 01:33:34 +0200 (Sun, 08 Sep 2002) $
 *
 * Standard library for Gont, String module implementation.
 */

open String;

string concat(string sep, <string>list lst)
{
    let ca_lst = List::map(to_char_array, lst);
    return of_char_array(Array::concat_sep(to_char_array(sep), ca_lst));
}

void iter(*(void (char)) f, string s)
{
    int sz = length(s);
    for (int i = 0; i < sz; i++)
	f(get(s, i));
}

extern int String___index_from(string s, int pos, char c);
extern int String___rindex_from(string s, int pos, char c);

int index_from(string s, int pos, char c)
{
    int ret = String___index_from(s, pos, c);

    if (ret == -2)
	raise Invalid_argument("String::index_from");
    if (ret == -1)
	raise Not_found;
	
    return ret;
}

int rindex_from(string s, int pos, char c)
{
    int ret = String___rindex_from(s, pos, c);

    if (ret == -2)
	raise Invalid_argument("String::rindex_from");
    if (ret == -1)
	raise Not_found;
	
    return ret;
}

bool contains_from(string s, int pos, char c)
{
    int ret = String___index_from(s, pos, c);

    if (ret == -2)
	raise Invalid_argument("String::contains_from");
    
    return ret >= 0;
}

bool rcontains_from(string s, int pos, char c)
{
    int ret = String___rindex_from(s, pos, c);

    if (ret == -2)
	raise Invalid_argument("String::rcontains_from");
    
    return ret >= 0;
}

int index(string s, char c)
{
    return index_from(s, 0, c);
}

int rindex(string s, char c)
{
    return rindex_from(s, length(s) - 1, c);
}

bool contains(string s, char c)
{
    return contains_from(s, 0, c);
}

extern int String___index_str_from(string, int, string);

int index_str_from(string s, int pos, string c)
{
    int ret = String___index_str_from(s, pos, c);
    
    if (ret == -2)
	raise Invalid_argument("String::index_str_from");
    
    if (ret == -1)
	raise Not_found;
    
    return ret;
}

bool contains_str_from(string s, int pos, string c)
{
    int ret = String___index_str_from(s, pos, c);
    
    if (ret == -2)
	raise Invalid_argument("String::contains_str_from");
    
    return ret != -1;
}

int index_str(string s, string c)
{
    return index_str_from(s, 0, c);
}

bool contains_str(string s, string c)
{
    return contains_str_from(s, 0, c);
}

string capitalize(string s)
{
    return make(1, Unitype::to_uppercase(get(s, 0))) + sub(s, 1, length(s) - 1);
}

string uncapitalize(string s)
{
    return make(1, Unitype::to_lowercase(get(s, 0))) + sub(s, 1, length(s) - 1);
}

<string>list split(string ssep, string ss)
{
    <string>list split_rec(int pos, string sep, string s)
    {
	if (contains_str_from(s, pos, sep)) {
	    int idx = index_str_from(s, pos, sep);
	    if (idx == pos && length(sep) == 0) {
		idx++;
		if (idx > length(s))
		    return Nil;
	    }
	    let x = sub(s, pos, idx - pos) in
	    let rest = split_rec(idx + length(sep), sep, s) in
	    return Cons(x, rest);
	} else {
	    return List::cons1(sub(s, pos, length(s) - pos));
	}
    }

    return split_rec(0, ssep, ss);
}

string uppercase(string s)
{
    <char>Array::t ret = Array::empty(length(s));
    iter(fun (c) (Array::append(ret, Unitype::to_uppercase(c))), s);
    return of_char_array(ret);
}

string lowercase(string s)
{
    <char>Array::t ret = Array::empty(length(s));
    iter(fun (c) (Array::append(ret, Unitype::to_lowercase(c))), s);
    return of_char_array(ret);
}

// vim:sts=4:sw=4
